/******************************************************************************
 *
 * "..and sky full of stars"
 * Copyright 2006 Jari Komppa
 * for ludum dare 48h game design contest #8
 * 
 ******************************************************************************
 *
 * License
 *
 *  This software is provided 'as-is', without any express or implied
 *  warranty.    In no event will the authors be held liable for any damages
 *  arising from the use of this software.
 * 
 *  Permission is granted to anyone to use this software for any purpose,
 *  including commercial applications, and to alter it and redistribute it
 *  freely, subject to the following restrictions:
 *
 *  1. The origin of this software must not be misrepresented; you must not
 *     claim that you wrote the original software. If you use this software
 *     in a product, an acknowledgment in the product documentation would be
 *     appreciated but is not required.
 *  2. Altered source versions must be plainly marked as such, and must not be
 *     misrepresented as being the original software.
 *  3. This notice may not be removed or altered from any source distribution.
 *
 * (eg. same as ZLIB license)
 *
 ******************************************************************************
 *
 * Requires SDL 1.2.x (1.2.9 used) and FMOD 3.7x (3.75 used) to compile.
 *
 ******************************************************************************/

/****************************************************************************** 
 *
 * Quick game done in ~10 hours
 * here be dragons
 * consider yourself warned.
 *
 ******************************************************************************/

// master disable for audio code; useful if you either don't want audio
// or are porting to a new platform without fmod
#define AUDIO_ENABLED

///////////////////////////////////////////////////////////////////////////////

#include <stdlib.h>
#include <string.h>
#include <math.h>
#include "SDL.h"
#ifdef AUDIO_ENABLED
#include "fmod.h"
#endif

///////////////////////////////////////////////////////////////////////////////

// msvc warning disable. 
#ifdef _MSC_VER
#pragma warning(disable:4244) // float = int
#endif

///////////////////////////////////////////////////////////////////////////////

#ifdef AUDIO_ENABLED
FSOUND_SAMPLE *s_pickup, *s_shadowdie, *s_groundcollision, *s_buildingdie;
FMUSIC_MODULE *s_midi;
#endif

///////////////////////////////////////////////////////////////////////////////

// the title (both window title and main menu)
#define TITLE "..and the sky full of stars 1.0"

///////////////////////////////////////////////////////////////////////////////

// some constants.. there are lots of magic numbers throughout 
// the code though.
#define STARS 100
#define BUILDINGS 20
#define EVIL 100
#define PARTICLE 1000

///////////////////////////////////////////////////////////////////////////////

// we're using three surfaces: sceen, font, and the help screen 
// (which is only a static 640x480 image)
SDL_Surface *screen;
SDL_Surface *font;
SDL_Surface *help;

///////////////////////////////////////////////////////////////////////////////

// game state tells what state we're in. Basically,
// menu -> help 
// menu -> begin 
// help -> menu
// begin -> ingame 
// ingame -> end 
// ingame -> over
// over -> menu
// ..although any state can return to menu via esc.
int gamestate;
// scheduled end of a state. All states except menu, help and ingame
// are timed and end at some time (2-6 secs if I remember right)
int stateendtick;
// options: are music and sound effects enabled
int op_music = 1;
int op_sfx = 1;

///////////////////////////////////////////////////////////////////////////////

// All the states the game can be in.
enum gamestates
{
	STATE_MENU,
	STATE_HELP,
	STATE_STAGE_BEGIN,
	STATE_INGAME,
	STATE_STAGE_END,
	STATE_GAME_OVER
};

///////////////////////////////////////////////////////////////////////////////

// Ingame related globals

// Number of buildings left
int buildings;
// Current score
int score;
// Unspawned monsters
int monsters_to_go;
// Total monsters for this stage 
// (needed for shrinking the gate 
// as monsters get spawned)
int stagemonsters;
// Current stage
int stage;
// Peak stars in the swarm, used to give bonus to player
int peakswarm;
// Current number of monsters out
int monsters_out; 
// Physics ticks 'till next monster spawn
int nextmonster;
// Currently collected (active) stars
int collected;


///////////////////////////////////////////////////////////////////////////////

// interface-related globals

// Current mouse coordinates
int mousex, mousey;
// World coordinates, i.e. the offset of the top left corner,
// or thereabouts, because one of these is reversed. Whoops.
float worldx, worldy;
// Last tick, used for 'physics tick' calculation (framerate-free rendering)
int lasttick;
// Active menu item. Now, this is a bit confusing. The currently active
// item is detected in rendering code, but the activation code is in the
// event handler.
int activemenu;

///////////////////////////////////////////////////////////////////////////////
// Structures

// Sigle particle. All of the particle effects (both of them)
// are done with the same code and structure
struct Particle
{
	float x, y;		// position
	float xv, yv;   // velocity
	float xi, yi;   // acceleration
	float size;     // size
	int age;        // physics ticks left to live
	int color;      // color
	int active;     // active flag
};

// buildings don't move, and stick to the ground..
struct Building
{
	float x, w, h; // position, width, height
	int color;     // color
	int active;    // active flag
};

// stars are a bit different in the way that if they're inactive,
// they're still visible. When the stars die, their x position is
// set to negative, which just means that the player can't possibly
// reach them.
struct Star
{
	float x,y,xv,yv; // position, velocity
	int active;      // active (moving) flag
};

// the evils stick to the ground and move to one direction at constant speed.
struct Evil
{
	float x,h,speed; // position, height (width = h/3), speed
	int active;      // active flag
};


///////////////////////////////////////////////////////////////////////////////

// arrays of structures

Star stars[STARS];
Building building[BUILDINGS];
Evil evil[EVIL];
Particle particle[PARTICLE];


///////////////////////////////////////////////////////////////////////////////

// text printing routines

// This is the most simple, and fastest to implement printer that I've
// ever done.

// Draw a single character.
// Characters are on top of each other in the font image, in ASCII order,
// so all this routine does is just set the coordinates for the character
// and use SDL to blit out.
void drawchar(char ch, int x, int y)
{
	SDL_Rect src, dst;
	src.w = 14;
	src.h = 24;
	src.x = 0;
	src.y = (ch - 32) * 24;
	dst.w = 14;
	dst.h = 24;
	dst.x = x;
	dst.y = y;
	SDL_BlitSurface(font, &src, screen, &dst);
}

// Draw the string. Characters are fixed width, so this is also
// deadly simple.
void drawstring(char * string, int x, int y)
{
	while (*string)
	{
		drawchar(*string,x,y);
		x += 14;
		string++;
	}
}

///////////////////////////////////////////////////////////////////////////////

// Spawn a particle. Instead of looking for a free particle slot, choose one
// at random and if it's busy, don't spawn a particle. Good enough.
void spawn(float x, float y, float xv, float yv, float xi, float yi, int color, int age, float size)
{
	int i = rand() % PARTICLE;
	if (particle[i].active)
		return;
	particle[i].active = 1;
	particle[i].x = x;
	particle[i].y = y;
	particle[i].xv = xv;
	particle[i].yv = yv;
	particle[i].xi = xi;
	particle[i].yi = yi;
	particle[i].age = age;
	particle[i].color = color;
	particle[i].size = size;
}

///////////////////////////////////////////////////////////////////////////////

// Initial initialization routine
void init()
{

#ifdef AUDIO_ENABLED
	// Init fmod
	if (FSOUND_GetVersion() < FMOD_VERSION)
		exit(0);
	if (!FSOUND_Init(32000, 64, 0))
		exit(0);
	
	// Load sounds
	s_pickup = FSOUND_Sample_Load(FSOUND_UNMANAGED, "pickup.wav", FSOUND_NORMAL, 0, 0);
	s_shadowdie = FSOUND_Sample_Load(FSOUND_UNMANAGED, "shadowdie.wav", FSOUND_NORMAL, 0, 0);
	s_groundcollision = FSOUND_Sample_Load(FSOUND_UNMANAGED, "ground.wav", FSOUND_NORMAL, 0, 0);
	s_buildingdie = FSOUND_Sample_Load(FSOUND_UNMANAGED, "buildingdie.wav", FSOUND_NORMAL, 0, 0);
	
	// Load mid, set its volume, loop on and play.
	s_midi = FMUSIC_LoadSong("tense.mid");
    FMUSIC_SetMasterVolume(s_midi, 128);
    FMUSIC_SetLooping(s_midi, 1);
    FMUSIC_PlaySong(s_midi);

#endif

	// Load font and help screen graphics.

	SDL_Surface *temp = SDL_LoadBMP("font14x24.bmp");
	font = SDL_ConvertSurface(temp, screen->format, SDL_SWSURFACE);
	SDL_FreeSurface(temp);
	
	// Seeing through the black areas might be a good idea..
	SDL_SetColorKey(font, SDL_SRCCOLORKEY, 0);  

	temp = SDL_LoadBMP("help.bmp");
	help = SDL_ConvertSurface(temp, screen->format, SDL_SWSURFACE);
	SDL_FreeSurface(temp);

	// Initially set to start from stage 5. If that's too hard, player
	// can always drop the stage lower (or set it higher if it's too easy).
	stage = 5;

	// and we'll start from the menu.
	gamestate = STATE_MENU;
}

// Reset for each stage.
void resetstage()
{
	int i;
	// Make sure we don't have any monsters alive
	for (i = 0; i < EVIL; i++)
	{
		evil[i].active = 0;
	}

	// re-set the stars in new random places
	for (i = 0; i < STARS; i++)
	{
		stars[i].x = rand()%2048;
		stars[i].y = rand()%600 - 488;
		stars[i].xv = 0;
		stars[i].yv = 0;
		stars[i].active = 0;
	}

	// same for buildings, plus the positions of the buildings
	// depend a bit on the stage (later levels -> wider city)
	for (i = 0; i < BUILDINGS; i++)
	{
		building[i].x = rand() % (600 + stage * 30);
		building[i].w = rand() % 20 + 10;
		building[i].h = rand() % 40 + 20;
		building[i].active = (i < buildings);
		// colors could be prettier, but completely random will do..
		building[i].color = rand() << 15 | rand();
	}
	// position "camera" near the middle..
	worldx = (2048-640)/2;
	worldy = 0;
	// and reposition mouse too
	SDL_WarpMouse(320,240);
	// number of monsters depends on the stage.
	// Progression: 5 8 13 20 29 40 53 68 80 80 80 80 80..
	// cap at 80 since there are only 100 stars in the sky
	monsters_to_go = stage * stage + 4;
	if (monsters_to_go > 80)
		monsters_to_go = 80;
	// store the total number of monsters for the gate animation
	stagemonsters = monsters_to_go;
	// reset bunch of variables
	monsters_out = 0;
	collected = 0;
	peakswarm = 0;
	// move on to begin stage, timed at 2 secs
	gamestate = STATE_STAGE_BEGIN;
	stateendtick = SDL_GetTicks() + 2000;
	// set last tick to current so that we don't run physics for ages
	lasttick = SDL_GetTicks();
	// and spawn first monster in 250 ticks. 
	// this doesn't actually work, since we're spawning a monster every time
	// there are no monsters around.
	nextmonster = 250;
}

// reset the game, when a new game starts
void reset()
{
	score = 0;
	// start with full house set
	buildings = BUILDINGS;
	resetstage();
}

// this function is used for almost all of the graphics in the game =)
// SDL handles the clipping etc. so this is pretty convenient..
void drawrect(int x, int y, int w, int h, int color)
{
	SDL_Rect r;
	r.x = x;
	r.y = y;
	r.w = w;
	r.h = h;
	SDL_FillRect(screen, &r, color);
}

// Collision of star vs. monster. 
int collision(Star &s, Evil &e)
{
	if (e.x < s.x &&
		e.x + e.h/3 > s.x &&
		440 - e.h < s.y)
		return 1;
	return 0;
}

// Collision of building vs. monster. 
// Overloaded for no apparent reason. Both of the collision checks
// are also only used in one place, and the code is pretty simple..
int collision(Building &b, Evil &e)
{
	if (b.x + b.w > e.x)
		return 1;
	return 0;
}

// Here's where we spend most of the time; the rendering function.
void render()
{   
	int i;
	int tick = SDL_GetTicks();

	// If in help state, just output the help screen and return.
	if (gamestate == STATE_HELP)
	{
		SDL_Rect src, dst;
		src.w = 640;
		src.h = 480;
		src.x = 0;
		src.y = 0;
		dst.w = 640;
		dst.h = 480;
		dst.x = 0;
		dst.y = 0;
		SDL_BlitSurface(help, &src, screen, &dst);
		// Tell SDL to update the whole screen
	    SDL_UpdateRect(screen, 0, 0, 640, 480);    
		SDL_Delay(10);
		return;
	}

	// Some states timeout; we'll handle them here.
	if (gamestate != STATE_INGAME && 
		gamestate != STATE_MENU && 
		stateendtick < tick)
	{
		switch (gamestate)
		{
		case STATE_STAGE_BEGIN:
			gamestate = STATE_INGAME;
			break;
		case STATE_STAGE_END:
			stage++;
			resetstage();
			break;
		case STATE_GAME_OVER:
			reset();
			gamestate = STATE_MENU;			
			break;
		}
	}

	// Check whether we're running over 50hz, and if so,
	// sleep for a bit.
	if (lasttick >= tick)
	{
		// going too fast..
		SDL_Delay(1);
		return;
	}

	// The "physics loop". This code is rendered at a 
	// stable 50hz even if the actual frame rate is lower.
	while (lasttick < tick)
	{
		int i, j, k;
		// move the world coordinates based on where the cursor is on screen
		float dx = ((mousex - 320) / 320.0f) * 2.5; 
		float dy = ((mousey - 240) / 240.0f) * 2.5;
		worldx += dx * dx * dx;
		worldy -= dy * dy * dy;
		// make sure the coordinates are still sane
		if (worldy > 1024-480) worldy = 1024-480;
		if (worldy < 0) worldy = 0;
		if (worldx < 0) worldx = 0;
		if (worldx > 2048-640) worldx = 2048 -640;

		if (gamestate == STATE_INGAME)
		{
			// spawn new monster based on timer, and make sure there's a monster
			// out there to remove the problem of player having to wait on the gate
			// with a huge swarm of stars for the rest of the monsters to come out..
			nextmonster--;
			if (nextmonster <= 0 || monsters_out == 0)
			{
				// spawn faster on later levels, but not too fast
				nextmonster = 100 - stage * 5;
				if (nextmonster <= 20) nextmonster = 10;
				nextmonster = 5 + rand()%(nextmonster);

				if (monsters_to_go != 0)
				{
					// same logic here as with the particles: pick a random slot, and
					// if the slot is in use, just don't spawn.
					i = rand() % EVIL;
					if (evil[i].active == 0)
					{
						evil[i].active = 1;
						evil[i].x = 2048-200;
						evil[i].h = rand() % 80+40;
						// speed depends a bit on stage; later stage monsters are faster
						evil[i].speed = rand() % (stage * 10) / 100.0f + 0.50f;
						monsters_out++;
						monsters_to_go--;
					}
				}
			}

			// move the monsters
			for (i = 0; i < EVIL; i++)
			{
				if (evil[i].active)
					evil[i].x -= evil[i].speed;
			}
		}

		// animate the stars even in non-ingame state, so that
		// things don't just stop stupidly on end of stage.
		for (i = 0; i < STARS; i++)
		{
			// the swarming of the stars is based on rather tweaked
			// algorithm that is not based on just about anything..
			float dx = ((mousex + worldx)-stars[i].x);
			float dy = ((mousey - worldy)-stars[i].y);
			float dist = sqrt(dx*dx+dy*dy);
			if (stars[i].active)
			{
				stars[i].xv += (1/dist) * dx;
				stars[i].yv += (1/dist) * dy;
				if (dist > 32)
				{
					stars[i].xv *= 0.99f;
					stars[i].yv *= 0.99f;
				}
				stars[i].x += stars[i].xv;
				stars[i].y += stars[i].yv;
				
				// collide with the ground
				if (stars[i].y > 440)
				{
					// bounce
					stars[i].yv = -stars[i].yv * 0.9f;
					stars[i].xv *= 0.9f;
					stars[i].y += stars[i].yv * 2;
#ifdef AUDIO_ENABLED
					if (op_sfx)
						FSOUND_PlaySound(FSOUND_FREE, s_groundcollision);
#endif
					// force the stars off ground
					if (stars[i].y > 440)
						stars[i].y = 440;
					
					// spawn 4 particles on collision
					for (k = 0; k < 4; k++)
					{
						spawn(
							stars[i].x, 
							440, 
							stars[i].xv*0.5+(rand()%100-50)/50.0f,
							stars[i].yv*0.2-2-(rand()%100)/50.0f,
							0,
							0.5,
							0xffffff,
							rand()%100,
							4);
					}
				}
			}
			else
			{
				if (gamestate == STATE_INGAME)
				{
					// pick up new stars (only in ingame mode)
					// the pick-up distance depends on currently picked-up stars
					// 0 -> 6
					// 8 -> 14
					// 20 -> 18
					// 100 -> 34
					if (dist < sqrt((float)collected*8)+6)
					{
						stars[i].active = 1;
						collected++;
						if (collected > peakswarm)
							peakswarm = collected;
						score += 1;
#ifdef AUDIO_ENABLED
						if (op_sfx)
							FSOUND_PlaySound(FSOUND_FREE, s_pickup);
#endif
					}
				}
			}
		}

		// check for star-evil collision
		for (i = 0; i < STARS; i++)
		{
			if (stars[i].active)
			{
				for (j = 0; j < EVIL; j++)
				{
					if (evil[j].active)
					{
						if (collision(stars[i],evil[j]))
						{
#ifdef AUDIO_ENABLED
							if (op_sfx)
								FSOUND_PlaySound(FSOUND_FREE, s_shadowdie);
#endif
							evil[j].active = 0;
							stars[i].active = 0;
							// since there's no "this star has been used and
							// won't be needed anymore" flag, we'll just
							// move the star outside the accessible world..
							stars[i].x = -1024; 
							collected--;
							score += 5;
							monsters_out--;
							// monster dies, spawn some particles.
							for (k = 0; k < 16; k++)
							{
								spawn(
									evil[j].x + rand() % 32, 
									420 - rand()%32, 
									((rand()%100)-50)/200.0f,
									0,
									0,
									-(rand()%100/500.0f+0.01),
									0x000000,
									rand()%200,
									12);
							}
						}
					}
				}
			}
		}

		// check for building-monster collision
		if (gamestate == STATE_INGAME)
		{
			for (i = 0; i < EVIL; i++)
			{
				if (evil[i].active)
				{
					for (j = 0; j < BUILDINGS; j++)
					{
						if (building[j].active)
						{
							if (collision(building[j],evil[i]))
							{
								evil[i].active = 0;
								building[j].active = 0;
								monsters_out--;
								buildings--;
#ifdef AUDIO_ENABLED
								if (op_sfx)
									FSOUND_PlaySound(FSOUND_FREE, s_buildingdie);
#endif
								// monster eats building,
								// spawn some particles
								// (pretty much same effect as the monster dies one)
								for (k = 0; k < 16; k++)
								{
									spawn(
										evil[i].x + rand() % 32, 
										420 - rand()%32, 
										((rand()%100)-50)/200.0f,
										0,
										0,
										-(rand()%100/500.0f+0.01),
										0x330000,
										rand()%200,
										12);
								}
							}
						}
					}
				}
			}

			// check for end of stage..
			if (monsters_to_go <= 0 &&
				monsters_out <= 0)
			{
				// set stage end state for 4 seconds
				gamestate = STATE_STAGE_END;
				stateendtick = SDL_GetTicks() + 4000;				
				// reward player
				score += buildings * 25;
				score += peakswarm * 5;
			}

			// check for game over
			if (buildings <= 0)
			{
				// set game over state for 4 seconds
				gamestate = STATE_GAME_OVER;
				stateendtick = SDL_GetTicks() + 4000;
			}
		}		

		// update particles. Particles accelerate, collide with the ground,
		// and die after their lifespan is over.
		// All particles are spawned with random age span, so they will disappear
		// "randomly", not needing a fadeout.
		for (i = 0; i < PARTICLE; i++)
		{
			if (particle[i].active)
			{
				particle[i].xv += particle[i].xi;
				particle[i].yv += particle[i].yi;
				particle[i].x += particle[i].xv;
				particle[i].y += particle[i].yv;
				if (particle[i].y > 440)
				{
					particle[i].y = 440;
					particle[i].yv = -particle[i].yv;
				}
				particle[i].age--;
				if (particle[i].age <= 0)
					particle[i].active = 0;				
			}
		}

		// 50hz, please.
		lasttick += 1000/50;
	}

/////////////////////////////////////////////////////////////////////
// rendering
/////////////////////////////////////////////////////////////////////

	// blue hue for background
	for (i = 0; i < 480; i++)
		drawrect(0, i, 640, 1, 0x100 - (((int)(worldy)-i+480) / 4));

	// some smaller stars for the backdrop
	// (using a pseudorandom noise function)
	unsigned int rr1 = 0xc0cac01a;
	unsigned int rr2 = 0x7aa71ee7;
	for (i = 0; i < 1024; i++)
	{
		rr1 = (rr1 >> 3) | (rr1 << (32 - rr1));
		int t = rr2;
		int xx = (rr2) % 2048;
		rr1 ^= 0x15;
		rr2 = rr1 ^ rr2;
		rr1 = t;
		int yy = (rr2) % 800 - 540;
		drawrect(xx - worldx,
				 yy + worldy,1,1,0xffffff);
	}

	// ground
	drawrect(0,(worldy)+440,640,100,0x11cc22);

	// the gate
	int pulse = sin(tick * 0.00512347)*8;
	int gatesize = 8 + (monsters_to_go * 100) / stagemonsters;
	if (monsters_to_go != 0)
		drawrect(2048-200-worldx-pulse/2,440-gatesize+worldy-pulse,60+pulse,gatesize+pulse,0);


	// particle fx
	for (i = 0; i < PARTICLE; i++)
	{
		if (particle[i].active)
		{
			drawrect(particle[i].x - worldx,
				     particle[i].y + worldy,
					 particle[i].size,
					 particle[i].size,
					 particle[i].color);
		}
	}

	// buildings
	for (i = 0; i < BUILDINGS; i++)
	{
		if (building[i].active)
		{
			drawrect(building[i].x - worldx,
					440-building[i].h + worldy,
					building[i].w,
					building[i].h,
					building[i].color);
		}
	}


	// stars in layers.. all darker areas first, then brigher, 
	// to avoid visual glitches.
	for (i = 0; i < STARS; i++)
	{
		drawrect((int)(stars[i].x - worldx)-6,
				 (int)(stars[i].y + worldy)-6,13,13,0x333333);
	}
	for (i = 0; i < STARS; i++)
	{
		drawrect((int)(stars[i].x - worldx)-4,
				 (int)(stars[i].y + worldy)-4,9,9,0x777777);
	}
	for (i = 0; i < STARS; i++)
	{
		drawrect((int)(stars[i].x - worldx)-2,
				 (int)(stars[i].y + worldy)-2,5,5,0xaaaaaa);
	}
	for (i = 0; i < STARS; i++)
	{
		if (stars[i].active)
		drawrect((int)(stars[i].x - worldx)-1,
				 (int)(stars[i].y + worldy)-1,3,3,0xffffff);
	}

	// then them monsters
	for (i = 0; i < EVIL; i++)
	{
		if (evil[i].active)
		{
			int pulse = sin((tick + i) * 0.012347)*4;
			float yscale = ((2048-200) - evil[i].x) / 50.0f;
			yscale = sqrt(yscale);
			if (yscale > 1.0) yscale = 1.0;

			int ht = (evil[i].h+pulse) * yscale;
			drawrect(evil[i].x-worldx-(pulse/2),
				     440-ht + worldy,
					 evil[i].h/3+pulse,
					 ht,0);

			// the eyes were .. fun .. to do.
			drawrect(evil[i].x-worldx + 2 -(pulse/2),
				     440-(evil[i].h+pulse - evil[i].h/6) * yscale +worldy,
					 2+pulse/2+evil[i].h/16,
					 (evil[i].h/12+pulse/2)*yscale,0x770000 + (((-pulse + 4)*16)<< 16));

			drawrect(evil[i].x-worldx + evil[i].h/3 - 4 - evil[i].h/16,
				     440-(evil[i].h - evil[i].h/6 + pulse)*yscale +worldy,
					 2+pulse/2+evil[i].h/16,
					 (evil[i].h/12+pulse/2)*yscale,0x770000 + (((-pulse + 4)*16)<< 16));

		}
	}


	// minimap
	if (gamestate != STATE_MENU)
	{

		// first the white outline
		drawrect(557,39,83,42,0xffffff);
		// ..background..
		drawrect(558,40,82,40,0x0033aa);
		// ..ground..
		drawrect(558,78,82,2,0x11cc22);
		// ..currently-visible-screen-area..
		drawrect(559+worldx / 25, 40-(worldy-560) / 25,24,16,0x555555);	

		// non-active stars (i.e. stuff to pick up)
		for (i = 0; i < STARS; i++)
		{
			if (stars[i].active == 0 && stars[i].x > 0)
			{
				drawrect(558+(stars[i].x / 25),66+(stars[i].y / 25),1,1,0xbbbbbb);
			}
		}

		// the monster positions
		for (i = 0; i < EVIL; i++)
		{
			if (evil[i].active == 1)
			{
				drawrect(559 + evil[i].x / 25, 70,4,8,0);
			}
		}

		// and the building positions
		for (i = 0; i < BUILDINGS; i++)
		{
			if (building[i].active)
			{
				drawrect(559 + building[i].x / 25, 78 - building[i].h/25, building[i].w / 25 + 1, building[i].h / 25 + 1, building[i].color);
			}
		}
	}

	// status texts
	char temp[200];
	if (gamestate != STATE_MENU)
	{
		sprintf(temp,"%08d",score);
		drawstring(temp,526,4);
		sprintf(temp,"Stage %02d",stage);
		drawstring(temp,6,4);
	}

	// we'll need to store the old menu action so that
	// we know when to play a sound when entering a new one.
	int oldaction;

	// ..and state-dependent texts
	switch (gamestate)
	{
	case STATE_STAGE_BEGIN:
		sprintf(temp,"Stage %02d",stage);
		drawstring(temp,260,200);
		drawstring("Get ready !!",250,240);			
		break;
	case STATE_STAGE_END:
		sprintf(temp,"Stage %02d done!",stage);
		drawstring(temp,260,200);
		drawstring("BONUS",300,260);
		sprintf(temp,"Buildings %d x 25 = %d", buildings, buildings * 25);
		drawstring(temp,200,300);
		sprintf(temp,"Peak swarm %d x 5 = %d", peakswarm, peakswarm * 5);
		drawstring(temp,200,340);
		break;
	case STATE_GAME_OVER:
		drawstring("Game over",300,200);
		sprintf(temp,"Score %d", score);
		drawstring(temp,300,260);
		break;
	case STATE_MENU:		
		drawstring(TITLE, 100,50);
		drawstring("[Start]", 260,150);
		drawstring("[<< Stage]", 100,200);
		drawstring("[Stage >>]", 360,200);
		sprintf(temp,"%02d",stage);
		drawstring(temp,290,200);
		drawstring("[Help]", 267,250);
		if (op_music)
			drawstring("[Music on ]", 67,250);
		else
			drawstring("[Music off]", 67,250);
		if (op_sfx)
			drawstring("[Sfx on ]", 397,250);
		else
			drawstring("[Sfx off]", 397,250);
		drawstring("[Quit]", 267,290);

		oldaction = activemenu;

		// mouse hit tests..

		if (mousex > 260 && mousex < 260+7*14 &&
			mousey > 150 && mousey < 150+24)
			activemenu = 1;
		else
		if (mousex > 100 && mousex < 100+10*14 &&
			mousey > 200 && mousey < 200+24)
			activemenu = 2;
		else
		if (mousex > 360 && mousex < 360+10*14 &&
			mousey > 200 && mousey < 200+24)
			activemenu = 3;
		else
		if (mousex > 268 && mousex < 268+6*14 &&
			mousey > 250 && mousey < 250+24)
			activemenu = 4;
		else
		if (mousex > 268 && mousex < 268+6*14 &&
			mousey > 290 && mousey < 290+24)
			activemenu = 5;
		else
		if (mousex > 68 && mousex < 68+11*14 &&
			mousey > 250 && mousey < 250+24)
			activemenu = 6;
		else
		if (mousex > 397 && mousex < 397+9*14 &&
			mousey > 250 && mousey < 250+24)
			activemenu = 7;
		else
			activemenu = 0;

		// note that the activemenu variable is global and
		// also used in the event handler.

		// play a sound if we're on a new item
		if (oldaction != activemenu && activemenu)
		{
#ifdef AUDIO_ENABLED
			if (op_sfx)
				FSOUND_PlaySound(FSOUND_FREE, s_groundcollision);
#endif
		}

		// ""hilight"" item we're on currently
		if (activemenu == 1) drawstring("[Start]", 261,150);
		if (activemenu == 2) drawstring("[<< Stage]", 101,200);
		if (activemenu == 3) drawstring("[Stage >>]", 361,200);
		if (activemenu == 4) drawstring("[Help]", 268,250);
		if (activemenu == 5) drawstring("[Quit]", 268,290);
		if (op_music)
			if (activemenu == 6) drawstring("[Music on ]", 68,250);
		else
			if (activemenu == 6) drawstring("[Music off]", 68,250);
		if (op_sfx)
			if (activemenu == 7) drawstring("[Sfx on ]", 398,250);
		else
			if (activemenu == 7) drawstring("[Sfx off]", 398,250);

		drawstring("For Ludum Dare #8 by Jari Komppa",100,350);
		drawstring("www.asshatdesigns.com",340,456);

		break;
	}

	// draw the cursor; size depends on the number of collected stars
	int cursorwidth = sqrt((float)collected*8)+6;
	// bigger black background outline
	drawrect(mousex-(cursorwidth+1),mousey-1,cursorwidth*2+4,4,0);
	drawrect(mousex-1,mousey-(cursorwidth+1),4,cursorwidth*2+4,0);
	
	drawrect(mousex-cursorwidth,mousey,cursorwidth*2+2,2,0x77aaff);
	drawrect(mousex,mousey-cursorwidth,2,cursorwidth*2+2,0x77aaff);

	// Tell SDL to update the whole screen
    SDL_UpdateRect(screen, 0, 0, 640, 480);    
}


// Entry point
int main(int argc, char *argv[])
{
	// Initialize SDL's subsystems - in this case, only video.
    if ( SDL_Init(SDL_INIT_VIDEO) < 0 ) 
	{
        fprintf(stderr, "Unable to init SDL: %s\n", SDL_GetError());
        exit(1);
    }

	// Register SDL_Quit to be called at exit; makes sure things are
	// cleaned up when we quit.
    atexit(SDL_Quit);
    
	// Attempt to create a 640x480 window with 32bit pixels. Default to
	// full screen, windowed if some parameter is given.
	if (argc > 1)
		screen = SDL_SetVideoMode(640, 480, 32, SDL_SWSURFACE);
	else
		screen = SDL_SetVideoMode(640, 480, 32, SDL_SWSURFACE | SDL_FULLSCREEN);
	
	// If we fail, return error.
    if ( screen == NULL ) 
	{
        fprintf(stderr, "Unable to set 640x480 video: %s\n", SDL_GetError());
        exit(1);
    }

	// we'll draw our own cursor, thank you
	SDL_ShowCursor(0);
	// set the caption for those windowed-mode players
	SDL_WM_SetCaption(TITLE,NULL);

	// reset first, then init, so that state is left to 'menu'.
	// (yep, I'm too lazy to set menu state here but not lazy
	// enough not to comment it later on).
	reset();
	init();

	// Main loop: loop forever.
    while (1)
    {
		// Render stuff
        render();

		// Poll for events, and handle the ones we care about.
        SDL_Event event;
        while (SDL_PollEvent(&event)) 
        {
            switch (event.type) 
            {
			case SDL_MOUSEMOTION:
				// store mouse coordinates for use in render function
				mousex = event.motion.x;
				mousey = event.motion.y;
				break;
			case SDL_MOUSEBUTTONUP:
				// only handle mouse buttons in menu (and help)
				if (gamestate == STATE_MENU)
				{
					// active menu item is set in render function
					int action = 1;
					switch (activemenu)
					{
					case 1:
						resetstage();
						break;
					case 2:
						stage--;
						if (stage < 1) stage = 1;
						break;
					case 3:
						stage++;
						if (stage > 25) stage = 25;
						break;
					case 4:
						gamestate = STATE_HELP;
						break;
					case 5:
#ifdef AUDIO_ENABLED
						FSOUND_Close();
#endif
						exit(0);
					case 6:
						op_music = 1 - op_music;
#ifdef AUDIO_ENABLED
						if (!op_music)
							FMUSIC_StopSong(s_midi);
						else
							FMUSIC_PlaySong(s_midi);
#endif
						break;
					case 7:
						op_sfx = 1 - op_sfx;
						break;
					default:
						// if no action is made, don't play a sound either
						action = 0;
					}
#ifdef AUDIO_ENABLED
					if (action)
						FSOUND_PlaySound(FSOUND_FREE, s_pickup);
#endif
				}
				else
				{
					// if not in menu state, and in help state,
					// return to menu state. (needs to be 'else' or
					// otherwise we go to menu and back immediately)
					if (gamestate == STATE_HELP)
						gamestate = STATE_MENU;
				}
				break;						

            case SDL_KEYUP:				
                if (event.key.keysym.sym == SDLK_ESCAPE)
				{
					// only key that is checked for is escape..
					// all states return to menu on esc, except
					// menu, which exits.
					if (gamestate == STATE_MENU)
					{
#ifdef AUDIO_ENABLED
					    FSOUND_Close();
#endif
						exit(0);
					}
					reset();
                    gamestate = STATE_MENU;
				}
                break;
            case SDL_QUIT:
				FSOUND_Close();
                return(0);
            }
        }
    }
    return 0;
}

